final def workDir = new File('.').canonicalFile
final def out = System.out
final def env = System.getenv()

final def isWindows = (System.getProperty('os.name') =~ /(?i)windows/).find()
final def ahptool = isWindows ? 'ahptool.cmd' : 'ahptool'

/**
 * A convenience method for running commands and optionally parsing the stdout of that command.
 * If closure is non-null, the closure will be passed the resultant {@link Process} and is expected to deal with all IO.
 * Otherwise, the process' stdOut and stdErr are forwarded to this scripts 'stdOut' and stdIn is untouched.
 * 
 * @param message an optional message to print prior to the commandline
 * @param command the command to be used as a String[]
 * @param closure an optional closure to deal with Process IO
 */
final def pb = new ProcessBuilder(['echo'] as String[]).directory(workDir)
final def runCommand = {def message, def command, def closure ->
    pb.command(command as String[])
    println()
    if (message) {
        println(message)
    }
    println("command: ${pb.command().join(' ')}")
    def proc = pb.start()
    if (closure) {
        closure(proc)
    }
    else {
        proc.consumeProcessOutput(out, out) // forward stdout and stderr
        proc.getOutputStream().close() // close stdin
    }
    proc.waitFor()
    if (proc.exitValue()) {
        throw new Exception("Command failed with exit code: "+proc.exitValue())
    }
}


//------------------------------------------------------------------------------
// GET ALL INPUT PARAMETERS 
//------------------------------------------------------------------------------

def ahptoolProc = [ahptool, "getStepProperties"].execute()
ahptoolProc.consumeProcessErrorStream(System.out);
def propertiesXml = ahptoolProc.text
ahptoolProc.waitFor()
if (ahptoolProc.exitValue()) {
    throw new Exception("Failed to get step properties from ahptool: " + propertiesXml)
}
def props = [:]
new XmlSlurper().parseText(propertiesXml).property.each{ propElem ->
    props[propElem.@name.text()] = propElem.text()
}

final def rubyVersionName = props['integration/versionName'] ? props['integration/versionName'] : 'Default'
final def versionLocation = props['integration/versionLocation']
final def rubyExe = isWindows ? 'ruby.exe' : 'ruby'
final def qualifiedRubyExe = versionLocation ? versionLocation + File.separator + rubyExe : rubyExe

final def scriptPath = props['rubyScriptFile']
final def scriptPathAbsolute = props['scriptPathAbsolute']
final def scriptLocation = scriptPathAbsolute ? scriptPath : (new File(workDir, scriptPath)).getAbsolutePath()
final def scriptText = props['rubyScriptText']
final def switches = props['switches']
final def arguments = props['arguments']

//------------------------------------------------------------------------------
//  MAIN PROCESS
//------------------------------------------------------------------------------

def command = [qualifiedRubyExe]

if (switches) {
   command += (switches.split(' ') as List)
}

command << scriptLocation

if (arguments) {
	command += (arguments.split("\n") as List)
}

runCommand('Run ruby script ' << scriptLocation << ' with Ruby: ' << rubyVersionName, command, null)



